@@ -16,6 +16,7 @@
 // under the License.
 
 use std::collections::HashMap;
+use std::path::PathBuf;
 use std::str::FromStr;
 use std::time::Duration;
 
@@ -89,7 +90,8 @@ impl Operator {
     }
 
     /// Open a file-like reader for the given path.
-    pub fn open(&self, path: String, mode: String) -> PyResult<File> {
+    pub fn open(&self, path: PathBuf, mode: String) -> PyResult<File> {
+        let path = path.to_string_lossy().to_string();
         let this = self.core.clone();
         if mode == "rb" {
             let r = this
@@ -109,18 +111,20 @@ impl Operator {
     }
 
     /// Read the whole path into bytes.
-    pub fn read<'p>(&'p self, py: Python<'p>, path: &str) -> PyResult<Bound<'p, PyAny>> {
-        let buffer = self.core.read(path).map_err(format_pyerr)?.to_vec();
+    pub fn read<'p>(&'p self, py: Python<'p>, path: PathBuf) -> PyResult<Bound<'p, PyAny>> {
+        let path = path.to_string_lossy().to_string();
+        let buffer = self.core.read(&path).map_err(format_pyerr)?.to_vec();
         Buffer::new(buffer).into_bytes_ref(py)
     }
 
     /// Write bytes into given path.
     #[pyo3(signature = (path, bs, **kwargs))]
-    pub fn write(&self, path: &str, bs: Vec<u8>, kwargs: Option<WriteOptions>) -> PyResult<()> {
+    pub fn write(&self, path: PathBuf, bs: Vec<u8>, kwargs: Option<WriteOptions>) -> PyResult<()> {
+        let path = path.to_string_lossy().to_string();
         let kwargs = kwargs.unwrap_or_default();
         let mut write = self
             .core
-            .write_with(path, bs)
+            .write_with(&path, bs)
             .append(kwargs.append.unwrap_or(false));
         if let Some(chunk) = kwargs.chunk {
             write = write.chunk(chunk);
@@ -135,30 +139,36 @@ impl Operator {
             write = write.cache_control(cache_control);
         }
 
-        write.call().map_err(format_pyerr)
+        write.call().map(|_| ()).map_err(format_pyerr)
     }
 
     /// Get current path's metadata **without cache** directly.
-    pub fn stat(&self, path: &str) -> PyResult<Metadata> {
+    pub fn stat(&self, path: PathBuf) -> PyResult<Metadata> {
+        let path = path.to_string_lossy().to_string();
         self.core
-            .stat(path)
+            .stat(&path)
             .map_err(format_pyerr)
             .map(Metadata::new)
     }
 
     /// Copy source to target.
-    pub fn copy(&self, source: &str, target: &str) -> PyResult<()> {
-        self.core.copy(source, target).map_err(format_pyerr)
+    pub fn copy(&self, source: PathBuf, target: PathBuf) -> PyResult<()> {
+        let source = source.to_string_lossy().to_string();
+        let target = target.to_string_lossy().to_string();
+        self.core.copy(&source, &target).map_err(format_pyerr)
     }
 
     /// Rename filename.
-    pub fn rename(&self, source: &str, target: &str) -> PyResult<()> {
-        self.core.rename(source, target).map_err(format_pyerr)
+    pub fn rename(&self, source: PathBuf, target: PathBuf) -> PyResult<()> {
+        let source = source.to_string_lossy().to_string();
+        let target = target.to_string_lossy().to_string();
+        self.core.rename(&source, &target).map_err(format_pyerr)
     }
 
     /// Remove all file
-    pub fn remove_all(&self, path: &str) -> PyResult<()> {
-        self.core.remove_all(path).map_err(format_pyerr)
+    pub fn remove_all(&self, path: PathBuf) -> PyResult<()> {
+        let path = path.to_string_lossy().to_string();
+        self.core.remove_all(&path).map_err(format_pyerr)
     }
 
     /// Create a dir at given path.
@@ -173,8 +183,9 @@ impl Operator {
     ///
     /// - Create on existing dir will succeed.
     /// - Create dir is always recursive, works like `mkdir -p`
-    pub fn create_dir(&self, path: &str) -> PyResult<()> {
-        self.core.create_dir(path).map_err(format_pyerr)
+    pub fn create_dir(&self, path: PathBuf) -> PyResult<()> {
+        let path = path.to_string_lossy().to_string();
+        self.core.create_dir(&path).map_err(format_pyerr)
     }
 
     /// Delete given path.
@@ -182,21 +193,34 @@ impl Operator {
     /// # Notes
     ///
     /// - Delete not existing error won't return errors.
-    pub fn delete(&self, path: &str) -> PyResult<()> {
-        self.core.delete(path).map_err(format_pyerr)
+    pub fn delete(&self, path: PathBuf) -> PyResult<()> {
+        let path = path.to_string_lossy().to_string();
+        self.core.delete(&path).map_err(format_pyerr)
+    }
+
+    /// Check given path is exists.
+    ///
+    /// # Notes
+    ///
+    /// - Check not existing path won't return errors.
+    pub fn exists(&self, path: PathBuf) -> PyResult<bool> {
+        let path = path.to_string_lossy().to_string();
+        self.core.exists(&path).map_err(format_pyerr)
     }
 
     /// List current dir path.
-    pub fn list(&self, path: &str) -> PyResult<BlockingLister> {
-        let l = self.core.lister(path).map_err(format_pyerr)?;
+    pub fn list(&self, path: PathBuf) -> PyResult<BlockingLister> {
+        let path = path.to_string_lossy().to_string();
+        let l = self.core.lister(&path).map_err(format_pyerr)?;
         Ok(BlockingLister::new(l))
     }
 
     /// List dir in flat way.
-    pub fn scan(&self, path: &str) -> PyResult<BlockingLister> {
+    pub fn scan(&self, path: PathBuf) -> PyResult<BlockingLister> {
+        let path = path.to_string_lossy().to_string();
         let l = self
             .core
-            .lister_with(path)
+            .lister_with(&path)
             .recursive(true)
             .call()
             .map_err(format_pyerr)?;
@@ -288,10 +312,11 @@ impl AsyncOperator {
     pub fn open<'p>(
         &'p self,
         py: Python<'p>,
-        path: String,
+        path: PathBuf,
         mode: String,
     ) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
 
         future_into_py(py, async move {
             if mode == "rb" {
@@ -315,8 +340,9 @@ impl AsyncOperator {
     }
 
     /// Read the whole path into bytes.
-    pub fn read<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+    pub fn read<'p>(&'p self, py: Python<'p>, path: PathBuf) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
         future_into_py(py, async move {
             let res: Vec<u8> = this.read(&path).await.map_err(format_pyerr)?.to_vec();
             Python::with_gil(|py| Buffer::new(res).into_bytes(py))
@@ -328,13 +354,14 @@ impl AsyncOperator {
     pub fn write<'p>(
         &'p self,
         py: Python<'p>,
-        path: String,
+        path: PathBuf,
         bs: &Bound<PyBytes>,
         kwargs: Option<WriteOptions>,
     ) -> PyResult<Bound<'p, PyAny>> {
         let kwargs = kwargs.unwrap_or_default();
         let this = self.core.clone();
         let bs = bs.as_bytes().to_vec();
+        let path = path.to_string_lossy().to_string();
         future_into_py(py, async move {
             let mut write = this
                 .write_with(&path, bs)
@@ -351,13 +378,14 @@ impl AsyncOperator {
             if let Some(cache_control) = &kwargs.cache_control {
                 write = write.cache_control(cache_control);
             }
-            write.await.map_err(format_pyerr)
+            write.await.map(|_| ()).map_err(format_pyerr)
         })
     }
 
     /// Get current path's metadata **without cache** directly.
-    pub fn stat<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+    pub fn stat<'p>(&'p self, py: Python<'p>, path: PathBuf) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
         future_into_py(py, async move {
             let res: Metadata = this
                 .stat(&path)
@@ -373,10 +401,12 @@ impl AsyncOperator {
     pub fn copy<'p>(
         &'p self,
         py: Python<'p>,
-        source: String,
-        target: String,
+        source: PathBuf,
+        target: PathBuf,
     ) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let source = source.to_string_lossy().to_string();
+        let target = target.to_string_lossy().to_string();
         future_into_py(py, async move {
             this.copy(&source, &target).await.map_err(format_pyerr)
         })
@@ -386,18 +416,21 @@ impl AsyncOperator {
     pub fn rename<'p>(
         &'p self,
         py: Python<'p>,
-        source: String,
-        target: String,
+        source: PathBuf,
+        target: PathBuf,
     ) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let source = source.to_string_lossy().to_string();
+        let target = target.to_string_lossy().to_string();
         future_into_py(py, async move {
             this.rename(&source, &target).await.map_err(format_pyerr)
         })
     }
 
     /// Remove all file
-    pub fn remove_all<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+    pub fn remove_all<'p>(&'p self, py: Python<'p>, path: PathBuf) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
         future_into_py(py, async move {
             this.remove_all(&path).await.map_err(format_pyerr)
         })
@@ -415,8 +448,9 @@ impl AsyncOperator {
     ///
     /// - Create on existing dir will succeed.
     /// - Create dir is always recursive, works like `mkdir -p`
-    pub fn create_dir<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+    pub fn create_dir<'p>(&'p self, py: Python<'p>, path: PathBuf) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
         future_into_py(py, async move {
             this.create_dir(&path).await.map_err(format_pyerr)
         })
@@ -427,17 +461,33 @@ impl AsyncOperator {
     /// # Notes
     ///
     /// - Delete not existing error won't return errors.
-    pub fn delete<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+    pub fn delete<'p>(&'p self, py: Python<'p>, path: PathBuf) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
         future_into_py(
             py,
             async move { this.delete(&path).await.map_err(format_pyerr) },
         )
     }
 
-    /// List current dir path.
-    pub fn list<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+    /// Check given path is exists.
+    ///
+    /// # Notes
+    ///
+    /// - Check not existing path won't return errors.
+    pub fn exists<'p>(&'p self, py: Python<'p>, path: PathBuf) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
+        future_into_py(
+            py,
+            async move { this.exists(&path).await.map_err(format_pyerr) },
+        )
+    }
+
+    /// List current dir path.
+    pub fn list<'p>(&'p self, py: Python<'p>, path: PathBuf) -> PyResult<Bound<'p, PyAny>> {
+        let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
         future_into_py(py, async move {
             let lister = this.lister(&path).await.map_err(format_pyerr)?;
             let pylister = Python::with_gil(|py| AsyncLister::new(lister).into_py_any(py))?;
@@ -447,8 +497,9 @@ impl AsyncOperator {
     }
 
     /// List dir in flat way.
-    pub fn scan<'p>(&'p self, py: Python<'p>, path: String) -> PyResult<Bound<'p, PyAny>> {
+    pub fn scan<'p>(&'p self, py: Python<'p>, path: PathBuf) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
         future_into_py(py, async move {
             let lister = this
                 .lister_with(&path)
@@ -465,10 +516,11 @@ impl AsyncOperator {
     pub fn presign_stat<'p>(
         &'p self,
         py: Python<'p>,
-        path: String,
+        path: PathBuf,
         expire_second: u64,
     ) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
         future_into_py(py, async move {
             let res = this
                 .presign_stat(&path, Duration::from_secs(expire_second))
@@ -484,10 +536,11 @@ impl AsyncOperator {
     pub fn presign_read<'p>(
         &'p self,
         py: Python<'p>,
-        path: String,
+        path: PathBuf,
         expire_second: u64,
     ) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
         future_into_py(py, async move {
             let res = this
                 .presign_read(&path, Duration::from_secs(expire_second))
@@ -503,10 +556,11 @@ impl AsyncOperator {
     pub fn presign_write<'p>(
         &'p self,
         py: Python<'p>,
-        path: String,
+        path: PathBuf,
         expire_second: u64,
     ) -> PyResult<Bound<'p, PyAny>> {
         let this = self.core.clone();
+        let path = path.to_string_lossy().to_string();
         future_into_py(py, async move {
             let res = this
                 .presign_write(&path, Duration::from_secs(expire_second))
