@@ -18,6 +18,7 @@
 import os
 from random import randint
 from uuid import uuid4
+from pathlib import Path
 
 import pytest
 from opendal.exceptions import NotFound
@@ -38,6 +39,21 @@ def test_sync_write(service_name, operator, async_operator):
     operator.delete(filename)
 
 
+@pytest.mark.need_capability("write", "delete", "stat")
+def test_sync_write_path(service_name, operator, async_operator):
+    size = randint(1, 1024)
+    filename = Path(f"test_file_{str(uuid4())}.txt")
+    content = os.urandom(size)
+    size = len(content)
+    operator.write(filename, content, content_type="text/plain")
+    metadata = operator.stat(filename)
+    assert metadata is not None
+    assert metadata.mode.is_file()
+    assert metadata.content_length == size
+
+    operator.delete(filename)
+
+
 @pytest.mark.asyncio
 @pytest.mark.need_capability("write", "delete", "stat")
 async def test_async_write(service_name, operator, async_operator):
@@ -54,6 +70,22 @@ async def test_async_write(service_name, operator, async_operator):
     await async_operator.delete(filename)
 
 
+@pytest.mark.asyncio
+@pytest.mark.need_capability("write", "delete", "stat")
+async def test_async_write_path(service_name, operator, async_operator):
+    size = randint(1, 1024)
+    filename = Path(f"test_file_{str(uuid4())}.txt")
+    content = os.urandom(size)
+    size = len(content)
+    await async_operator.write(filename, content)
+    metadata = await async_operator.stat(filename)
+    assert metadata is not None
+    assert metadata.mode.is_file()
+    assert metadata.content_length == size
+
+    await async_operator.delete(filename)
+
+
 @pytest.mark.need_capability("create_dir", "stat")
 def test_sync_create_dir(service_name, operator, async_operator):
     path = f"test_dir_{str(uuid4())}/"
